//
//  SqliteDataAdapter.cs
//
//  Author(s): Mars Tanumihardja  <m.tanu@gte.net>
//
//  Copyright (C) 2003  Mars Tanumihardja
//

using System;
using System.Diagnostics;
using System.Data;
using System.Data.Common;

namespace Mono.Data.SqliteClient
{
	public class SqliteDataAdapter : DbDataAdapter, IDbDataAdapter, IDataAdapter 
	{
		IDbCommand _SelectCommand;
		IDbCommand _InsertCommand;
		IDbCommand _UpdateCommand;
		IDbCommand _DeleteCommand;
		
		public SqliteDataAdapter() 
		{
		}

		public SqliteDataAdapter(SqliteCommand selectCommand) 
		{
			_SelectCommand = selectCommand;
		}

		public SqliteDataAdapter(string selectCommandText, string selectConnectionString) 
		{
			SqliteConnection conn = new SqliteConnection(selectConnectionString);
			_SelectCommand = new SqliteCommand(selectCommandText, conn);
		}

		public SqliteDataAdapter(string selectCommandText, SqliteConnection conn) 
		{
			_SelectCommand = new SqliteCommand(selectCommandText, conn);
		}

		void NotSupported()
		{
			throw new System.NotSupportedException();
		}

		void NotImpl()
		{
			throw new System.NotImplementedException();
		}

        protected override RowUpdatedEventArgs CreateRowUpdatedEvent(
					  DataRow dataRow,
					  IDbCommand command,
					  StatementType statementType,
					  DataTableMapping tableMapping)
		{
			Debug.WriteLine("SqliteDataReader.CreateRowUpdatedEvent", "SqliteDataAdapter"); 
			NotSupported();
			return null;
		}
		
	    protected override RowUpdatingEventArgs CreateRowUpdatingEvent(
						DataRow dataRow,
						IDbCommand command,
						StatementType statementType,
						DataTableMapping tableMapping) 
		{
			Debug.WriteLine("SqliteDataAdapter.CreateRowUpdatingEvent", "SqliteDataAdapter");  
			NotSupported();
			return null;
		}

		protected override void OnRowUpdated(RowUpdatedEventArgs args) 
		{
			Debug.WriteLine("SqliteDataAdapter.OnRowUpdated", "SqliteDataAdapter"); 
		}

		protected override void OnRowUpdating(RowUpdatingEventArgs args) 
		{
			Debug.WriteLine("SqliteDataAdapter.get_OnRowUpdating", "SqliteDataAdapter"); 
		}

		public IDbCommand InsertCommand
		{
			get 
			{
				Debug.WriteLine("SqliteDataAdapter.get_InsertCommand", "SqliteDataAdapter");  
				NotSupported();
				return _InsertCommand;
			}
			set 
			{
				Debug.WriteLine("SqliteDataAdapter.set_InsertCommand", "SqliteDataAdapter"); 
				NotSupported(); 
				_InsertCommand = value;
			}
		}

		public IDbCommand UpdateCommand
		{
			get 
			{
				Debug.WriteLine("SqliteDataAdapter.get_UpdateCommand", "SqliteDataAdapter");  
				NotSupported();
				return _UpdateCommand;
			}
			set 
			{
				Debug.WriteLine("SqliteDataAdapter.set_UpdateCommand", "SqliteDataAdapter");  
				NotSupported();
				_UpdateCommand = value;
			}
		}

		public IDbCommand DeleteCommand
		{
			get 
			{
				Debug.WriteLine("SqliteDataAdapter.get_DeleteCommand", "SqliteDataAdapter");  
				NotSupported();
				return _DeleteCommand;
			}
			set 
			{ 
				Debug.WriteLine("SqliteDataAdapter.set_DeleteCommand", "SqliteDataAdapter"); 
				NotSupported();
				_DeleteCommand = value;
			}
		}

		public IDbCommand SelectCommand
		{
			get 
			{ 
				Debug.WriteLine("SqliteDataAdapter.get_SelectCommand", "SqliteDataAdapter"); 
				return _SelectCommand;
			}
			set 
			{ 
				Debug.WriteLine("SqliteDataAdapter.set_SelectCommand", "SqliteDataAdapter"); 
				_SelectCommand = value;
			}
		}

		public override int Fill(DataSet dataSet) 
		{
			DataTable dataTable = new DataTable();
			SqliteDataReader reader = (SqliteDataReader)_SelectCommand.ExecuteReader();
			try 
			{
				bool columnsCreated = false;
				while (reader.Read()) 
				{
					if (!columnsCreated) 
					{
						for (int i=0; i<reader.FieldCount; i++) 
						{
							// shortcut here: instead of calling GetSchemaTable(), use
							// method GetName() and assume all fields typeof(string).
							string colName = reader.GetName(i);
							dataTable.Columns.Add(colName, typeof(string));
						}
						columnsCreated = true;
					}

					object[] fields = new object[reader.FieldCount];
					reader.GetValues(fields);
					dataTable.Rows.Add(fields);
				}
			}
			finally 
			{
				reader.Close();
			}
			dataSet.Tables.Add(dataTable);

			return dataTable.Rows.Count;
		}
	}
}
